# -*- coding: utf-8 -*-
"""
Created on Fri May 22 15:11:06 2022

@author: Somaiya M. Ghoraibi
"""


import surprise
import numpy as np
import pandas as pd
import sklearn
import matplotlib.pyplot as plt
import seaborn as sns 
from surprise import Reader 
from surprise import SVD,SVDpp, SlopeOne, NMF, NormalPredictor, KNNBaseline, KNNBasic, KNNWithMeans, KNNWithZScore, BaselineOnly, CoClustering
from surprise import Dataset
from surprise.model_selection import cross_validate
from surprise.model_selection import KFold
from surprise import accuracy




#--------------------------- 

#START RS



#For AmaZon

CSV_path='D:\\Sumaia\\Amazon\\AfteroptimizeratingFINALNEWFULLDataBase.csv'
df=pd.read_csv(CSV_path)
df.columns
df.shape[0]

#Rescale Implicit Rating similar to the baseline rating [1,5]
# 1. Implicit rating using sentiment word
s=[]
for i in range (df.shape[0]): 
    s.append(df['ImplicitTotalSentimentWord'][i])

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['NormalizedImplicitTotalSentimentWord']=s2

# 2. Implicit rating using Aspect
s=[]
for i in range (df.shape[0]): 
    s.append(df['ImplicitRating_Aspect'][i])

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['NormalizedImplicitRating_Aspect']=s2

# 3. Calcuate average rating using sentiment word and overall then Rescale the new rating into [1,5]

s=[]
for i in range (df.shape[0]): 
   s.append((df['ImplicitTotalSentimentWord'][i]+ df['overall'][i])/2)
   #s.append( (df['ImplicitRating_Aspect'][i]+ df['overall'][i])/2)

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['AverageNormalizedImplicitTotalSentimentWord']=s2


# 4. Calcuate average rating using aspect and overall then Rescale the new rating into [1,5]

s=[]
for i in range (df.shape[0]): 
  s.append( (df['ImplicitRating_Aspect'][i]+ df['overall'][i])/2)

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['AverageNormalizedImplicitRating_Aspect']=s2




#Each time we drop all the unused columns

df.drop(['reviewerName', 'helpful', 'reviewText',
       'summary', 'unixReviewTime', 'reviewTime',
       'ImplicitRating_Aspect', 
        'ImplicitTotalSentimentWord','overall', 'NormalizedImplicitRating_Aspect', #'NormalizedImplicitTotalSentimentWord',
        'AverageNormalizedImplicitRating_Aspect', 'AverageNormalizedImplicitTotalSentimentWord'
        
        ], axis=1, inplace=True)

#To reduce the dimensionality of the data set, and avoid running into “memory error”, we will filter out rarely rated books and rarely rating users.
min_movie_ratings = 20
filter_movies = df['ProductID'].value_counts() > min_movie_ratings
filter_movies = filter_movies[filter_movies].index.tolist()

min_user_ratings = 20
filter_users = df['reviewerID'].value_counts() > min_user_ratings
filter_users = filter_users[filter_users].index.tolist()

df_new = df[(df['ProductID'].isin(filter_movies)) & (df['reviewerID'].isin(filter_users))]
print('The original data frame shape:\t{}'.format(df.shape))
print('The new data frame shape:\t{}'.format(df_new.shape))


Users = df_new.reviewerID.nunique()
Products = df_new.ProductID.nunique()
print('The Users:\t{}'.format(Users))
print('The Products:\t{}'.format(Products))


#To load a data set from the above pandas data frame, we will use the load_from_df() method, we will also need a Reader object, and the rating_scale parameter must be specified. The data frame must have three columns, corresponding to the user ids, the item ids, and the ratings in this order. Each row thus corresponds to a given rating
reader = Reader(rating_scale=(1, 5))

#Here is the step for determining which rating will be used for integrating into the CF rating prediction algorithms
#Each time we use one

#data = Dataset.load_from_df(df_new[['reviewerID', 'ProductID', 'overall']], reader)
data = Dataset.load_from_df(df_new[['reviewerID', 'ProductID', 'NormalizedImplicitTotalSentimentWord']], reader)
#data = Dataset.load_from_df(df_new[['reviewerID', 'ProductID', 'NormalizedImplicitRating_Aspect']], reader)
#data = Dataset.load_from_df(df_new[['reviewerID', 'ProductID', 'AverageNormalizedImplicitTotalSentimentWord']], reader)
#data = Dataset.load_from_df(df_new[['reviewerID', 'ProductID', 'AverageNormalizedImplicitRating_Aspect']], reader)




# define a cross-validation iterator
kf = surprise.model_selection.split.KFold(n_splits=5, random_state=None, shuffle=False)

#Each time we use one of the following algorithms
#SVD,SVDpp, SlopeOne, NMF, NormalPredictor, KNNBaseline, KNNBasic, KNNWithMeans, KNNWithZScore, BaselineOnly, CoClustering


algo=SVD()


#sim_options = {'name':'cosine','user_based':True}
#algo = KNNBasic(k=50,sim_options=sim_options)
#algo = KNNBasic (k=50)
#algo = KNNWithMeans(k=50,sim_options=sim_options)
#algo = SVDpp ()
#algo = NMF()
#algo= SlopeOne()
#algo= CoClustering()
#algo= NormalPredictor()
#algo=KNNWithZScore(k=50)
#algo=BaselineOnly()

#algo = KNNWithZScore(k=50)
#algo=KNNBaseline(k=50)
#algo = KNNBasic (k=50,sim_options=sim_options)

#sim_options = {'user_based':False}
#sim_options = {'user_based':True}

#algo = KNNBaseline (k=50,sim_options=sim_options)
#sim_options = {'name': 'cosine'}#,'user_based':False}
#algo = KNNWithMeans (k=50,sim_options=sim_options)
#algo = KNNWithZScore (k=50,sim_options=sim_options)
#algo = KNNBaseline (k=50,sim_options=sim_options)
#algo = KNNBasic(k=50,sim_options=sim_options)





#Train and test , record the error metrics values : MAE,MSE, and RMSE


i=0
for trainset, testset in kf.split(data):
 
    i+=1
    print (i)
    # train and test algorithm.
    if (i<6):
      algo.fit(trainset)
      predictions = algo.test(testset)

    # Compute and print Root Mean Squared Error
      accuracy.mae(predictions, verbose=True)
      accuracy.mse(predictions, verbose=True)
      accuracy.rmse(predictions, verbose=True)


#-----------------------------------------------------------------
    
#For Yelp
CSV_path='C:\\Users\\Sumaia\\Desktop\\Programming For Yelp\\Yelp - Restaurant\\Implicit_Rating_Aspect\\FullDataSet_RatingsWithNormalizationAndAverag.csv'

df=pd.read_csv(CSV_path)
df.columns
df.shape[0]

#Rescale Implicit Rating similar to the baseline rating [1,5]
# 1. Implicit rating using sentiment word
s=[]
for i in range (df.shape[0]): 
    s.append(df['ImplicitTotalSentimentWord'][i])

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['NormalizedImplicitTotalSentimentWord']=s2

# 2. Implicit rating using Aspect
s=[]
for i in range (df.shape[0]): 
    s.append(df['ImplicitRating_Aspect'][i])

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['NormalizedImplicitRating_Aspect']=s2

# 3. Calcuate average rating using sentiment word and overall then Rescale the new rating into [1,5]

s=[]
for i in range (df.shape[0]): 
   s.append((df['ImplicitTotalSentimentWord'][i]+ df['overall'][i])/2)
   #s.append( (df['ImplicitRating_Aspect'][i]+ df['overall'][i])/2)

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['AverageNormalizedImplicitTotalSentimentWord']=s2


# 4. Calcuate average rating using aspect and overall then Rescale the new rating into [1,5]

s=[]
for i in range (df.shape[0]): 
  s.append( (df['ImplicitRating_Aspect'][i]+ df['overall'][i])/2)

original_min=min(s)
original_max=max(s)

New_min=1
New_max=5

s2=[]
for i in range (df.shape[0]):
    New_rating = New_min + [(New_max - New_min) * (s[i] - original_min)] / (original_max -original_min)
    s2.append(New_rating)
    
df['AverageNormalizedImplicitRating_Aspect']=s2


#Each time we drop all the unused columns 
    
df.drop(['reviewerID',  'helpful', 'reviewText', 'overall' ,
        'reviewTime', 'Aspects', 'brilliant_Aspect', 'Funny', 'Cool',     
        'NormalizedImplicitRating_Aspect',  
       'ImplicitRating_Aspect','ImplicitTotalSentimentWord',  # 'NormalizedImplicitTotalSentimentWord',
       'AverageNormalizedImplicitRating_Aspect',
       'AverageNormalizedImplicitTotalSentimentWord'
      ]
    
    , axis=1, inplace=True)


#To reduce the dimensionality of the data set, and avoid running into “memory error”, we will filter out rarely rated books and rarely rating users.
min_movie_ratings = 10
filter_movies = df['ProductID'].value_counts() > min_movie_ratings
filter_movies = filter_movies[filter_movies].index.tolist()

min_user_ratings = 10
filter_users = df['reviewerName'].value_counts() > min_user_ratings
filter_users = filter_users[filter_users].index.tolist()

#Yelp
df_new = df[(df['ProductID'].isin(filter_movies)) & (df['reviewerName'].isin(filter_users))]
print('The original data frame shape:\t{}'.format(df.shape))
print('The new data frame shape:\t{}'.format(df_new.shape))

Users = df_new.reviewerName.nunique()
Products = df_new.ProductID.nunique()
print('The Users:\t{}'.format(Users))
print('The Products:\t{}'.format(Products))



#Here is the step for determining which rating will be used for integrating into the CF rating prediction algorithms
#Each time we use one

#data = Dataset.load_from_df(df_new[['reviewerName', 'ProductID', 'overall']], reader)
#data = Dataset.load_from_df(df_new[['reviewerName', 'ProductID', 'NormalizedImplicitRating_Aspect']], reader)
#data = Dataset.load_from_df(df_new[['reviewerName', 'ProductID', 'NormalizedImplicitTotalSentimentWord']], reader)
#data = Dataset.load_from_df(df_new[['reviewerName', 'ProductID', 'AverageNormalizedImplicitTotalSentimentWord']], reader)
data = Dataset.load_from_df(df_new[['reviewerName', 'ProductID', 'AverageNormalizedImplicitRating_Aspect']], reader)



# define a cross-validation iterator
kf = surprise.model_selection.split.KFold(n_splits=5, random_state=None, shuffle=False)

#Each time we use one of the following algorithms
#SVD,SVDpp, SlopeOne, NMF, NormalPredictor, KNNBaseline, KNNBasic, KNNWithMeans, KNNWithZScore, BaselineOnly, CoClustering


algo=SVD()


#sim_options = {'name':'cosine','user_based':True}
#algo = KNNBasic(k=30,sim_options=sim_options)
#algo = KNNBasic (k=30)
#algo = KNNWithMeans(k=30,sim_options=sim_options)
#algo = SVDpp ()
#algo = NMF()
#algo= SlopeOne()
#algo= CoClustering()
#algo= NormalPredictor()
#algo=KNNWithZScore(k=30)
#algo=BaselineOnly()

#algo = KNNWithZScore(k=30)
#algo=KNNBaseline(k=30)
#algo = KNNBasic (k=30,sim_options=sim_options)

#sim_options = {'user_based':False}
#sim_options = {'user_based':True}

#algo = KNNBaseline (k=30,sim_options=sim_options)
#sim_options = {'name': 'cosine'}#,'user_based':False}
#algo = KNNWithMeans (k=30,sim_options=sim_options)
#algo = KNNWithZScore (k=30,sim_options=sim_options)
#algo = KNNBaseline (k=30,sim_options=sim_options)
#algo = KNNBasic(k=30,sim_options=sim_options)





#Train and test , record the error metrics values : MAE,MSE, and RMSE


i=0
for trainset, testset in kf.split(data):
 
    i+=1
    print (i)
    # train and test algorithm.
    if (i<6):
      algo.fit(trainset)
      predictions = algo.test(testset)

    # Compute and print Root Mean Squared Error
      accuracy.mae(predictions, verbose=True)
      accuracy.mse(predictions, verbose=True)
      accuracy.rmse(predictions, verbose=True)


#-----------------------------------





